--[[----------------------------------------------------------------------------
	Copyright 2007-2009 João Afonso Libório Cardoso
	All rights reserved
	
	MailNotifier is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License version 3 as published by the Free Software Foundation.
	MailNotifier is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License (<http://www.gnu.org/licenses/>) for more details.
---------------------------------------------------------------------------]]--

local MailNotifier = CreateFrame('Frame', 'MailNotifier')
local Indicator, Shine, WrongEvent, LoggingIn, NewMails, AtInbox
local L, Sets = MailNotifier_Locals

local AUCTION_OUTBID = ERR_AUCTION_OUTBID_S:gsub('%%s', '%.+')
local AUCTION_WON = ERR_AUCTION_WON_S:gsub('%%s', '%.+')


--[[ Startup ]]--

function MailNotifier:Load()
	Indicator = MiniMapMailFrame:CreateFontString('MailNotifierIndicator', 'OVERLAY')
	Indicator:SetFontObject('NumberFontNormal')
	Indicator:SetPoint('Center', MiniMapMailFrame, 'Center', -1, 1)
	Indicator:SetJustifyH('CENTER')

	Shine = MiniMapMailFrame:CreateTexture('MailNotifierShine', 'OVERLAY')
	Shine:SetTexture('Interface\\Calendar\\EventNotificationGlow')
	Shine:SetPoint('CENTER', MiniMapMailFrame, 'CENTER', -2, 2)
	Shine:SetHeight(35) Shine:SetWidth(35)
	Shine:SetBlendMode('ADD')
	Shine:Hide()
	
	self:SetScript('OnEvent', function(self, event, ...) self[event](self, ...) end)
	self:RegisterEvent('VARIABLES_LOADED')
	self:RegisterEvent('PLAYER_LOGOUT')
	self:RegisterEvent('PLAYER_ALIVE')
	self:RegisterEvent('PLAYER_DEAD')
	
	self:RegisterEvent('UPDATE_PENDING_MAIL')
	self:RegisterEvent('MAIL_INBOX_UPDATE')
	self:RegisterEvent('MAIL_CLOSED')
end


--[[ Saved Variables ]]--

function MailNotifier:VARIABLES_LOADED()
	MailNotifier_SV = MailNotifier_SV or {unread = 0, senders = {}}
	Sets = MailNotifier_SV
end

function MailNotifier:PLAYER_LOGOUT()
	self:SaveSenders(GetLatestThreeSenders())
end


--[[ Mail API ]]--

function MailNotifier:AddNewMail(new, silent)
	self:SetNumMails(self:GetNumMails() + new)
	if not silent then
		self:AlertUser()
	end
end

function MailNotifier:SetNumMails(value)
	Sets.unread = value
	self:UpdateIndicator()
end

function MailNotifier:GetNumMails()
	return Sets.unread
end


--[[ Senders API ]]--

function MailNotifier:SaveSenders(...)
	wipe(Sets.senders)
	for i = 1, select('#', ...) do
		Sets.senders[select(i, ...)] = i
	end
end

function MailNotifier:GetSavedSenders()
	return Sets.senders
end


--[[ Button API ]]--

function MailNotifier:UpdateIndicator()
	Indicator:SetText(self:GetNumMails() or '')
end

function MailNotifier:AlertUser()
	PlaySoundFile("Interface\\AddOns\\MailNotifier\\NewMail.wav")
	UIFrameFlash(Shine, 1, 1, 10, nil, 0.6)
end


--[[ Outdoor Events ]]--

function MailNotifier:PLAYER_ALIVE()
	LoggingIn = true
end

function MailNotifier:PLAYER_DEAD()
	LoggingIn = true
end

function MailNotifier:PLAYER_ENTERING_WORLD()
	WrongEvent = true
end

function MailNotifier:UPDATE_PENDING_MAIL()
	GetLatestThreeSenders() -- forces the "inbox update" event

	self:RegisterEvent('PLAYER_ENTERING_WORLD')
	self:RegisterEvent('CHAT_MSG_SYSTEM')
	self:UnregisterEvent('PLAYER_ALIVE')
	self:UnregisterEvent('PLAYER_DEAD')
	self:UpdateIndicator()

	function self:UPDATE_PENDING_MAIL() -- has a "startup" function and then changes to the basic one
		if WrongEvent or LoggingIn or AtInbox then
			WrongEvent = nil
			return
		end
		self:AddNewMail(1)
	end
end


--[[ Inbox Events ]]--

function MailNotifier:MAIL_INBOX_UPDATE()
	function self:MAIL_INBOX_UPDATE() -- ignores once if logged in
		self:SetNumMails(self:GetInboxNumMails())
		
		if not AtInbox then
			NewMails = self:GetNumMails()
			AtInbox = true
		elseif self:GetNumMails() ~= NewMails then
			WrongEvent = true
		end
	end
		
	if LoggingIn then
		if HasNewMail() then
			local NewSenders = {GetLatestThreeSenders()}
			local OldSenders = self:GetSavedSenders()
			local newMails, oldI, off = 0
			
			for newI, sender in pairs(NewSenders) do
				oldI = OldSenders[sender] or 0
					
				if newI >= oldI then
					off = newI - oldI
				else
					off = 3 - oldI + newI
				end
					
				newMails = max(newMails, off)
			end
			
			self:AddNewMail(newMails, newMails == 0)
		end
		LoggingIn = nil
	else
		self:MAIL_INBOX_UPDATE()
	end
end

function MailNotifier:MAIL_CLOSED()
	AtInbox = nil
end

function MailNotifier:GetInboxNumMails()
	local newMails = 0
	for i = 1, GetInboxNumItems() do
		if not select(9, GetInboxHeaderInfo(i)) then
			newMails = newMails + 1
		end
	end
	return newMails
end


--[[ System Messages ]]--

function MailNotifier:CHAT_MSG_SYSTEM(message)
	if strmatch(message, AUCTION_OUTBID) then
		self:UPDATE_PENDING_MAIL()
	elseif arg1 == ERR_AUCTION_REMOVED or strmatch(message, AUCTION_WON) then
		WrongEvent = true
	end
end


--[[ Tootlip ]]--

function MinimapMailFrameUpdate()
	local numMails = MailNotifier:GetNumMails()
	local senders = {GetLatestThreeSenders()}
	local title
	
	if numMails ~= 1 then
		title = L.HaveMails
	else
		title = L.HaveMail
	end
			
	if #senders > 0 then
		title = title .. L.From
	end
	
	GameTooltip:SetText(format(title, numMails))
	
	for i,sender in pairs(senders) do
		GameTooltip:AddLine(' - ' .. sender, 1, 1, 1)
	end
	
	GameTooltip:Show()
end


--[[ Call Addon ]]--

MailNotifier:Load()
--DevTools.EventTrace_SlashCommand("")
--[[ Utilizar CHANNEL_UI_UPDATE ou CHAT_MSG_CHANNEL_NOTICE para iniciar em vez de INBOX_UPDATE ]]--